/* automatically generated from xfce4-screensaver-preferences.ui */
#ifdef __SUNPRO_C
#pragma align 4 (xfce4_screensaver_preferences_ui)
#endif
#ifdef __GNUC__
static const char xfce4_screensaver_preferences_ui[] __attribute__ ((__aligned__ (4))) =
#else
static const char xfce4_screensaver_preferences_ui[] =
#endif
{
  "<?xml version=\"1.0\" encoding=\"UTF-8\"?><interface><requires lib=\"gt"
  "k+\" version=\"3.20\"/><requires lib=\"libxfce4ui-2\" version=\"4.12\"/"
  "><object class=\"GtkWindow\" id=\"fullscreen_preview_window\"><property"
  " name=\"can_focus\">True</property><property name=\"has_focus\">True</p"
  "roperty><property name=\"title\" translatable=\"yes\">Screensaver Previ"
  "ew</property><property name=\"modal\">True</property><property name=\"w"
  "indow_position\">center</property><property name=\"destroy_with_parent\""
  ">True</property><property name=\"skip_taskbar_hint\">True</property><pr"
  "operty name=\"skip_pager_hint\">True</property><property name=\"decorat"
  "ed\">False</property><child><placeholder/></child><child><object class="
  "\"GtkBox\" id=\"fullscreen_preview_vbox\"><property name=\"visible\">Tr"
  "ue</property><property name=\"can_focus\">False</property><property nam"
  "e=\"orientation\">vertical</property><child><object class=\"GtkBox\" id"
  "=\"fullscreen_preview_hbox\"><property name=\"visible\">True</property>"
  "<property name=\"can_focus\">False</property><property name=\"spacing\""
  ">6</property><child><object class=\"GtkImage\" id=\"fullscreen_preview_"
  "image\"><property name=\"visible\">True</property><property name=\"can_"
  "focus\">False</property><property name=\"margin_left\">5</property><pro"
  "perty name=\"margin_right\">5</property><property name=\"stock\">gtk-di"
  "alog-warning</property></object><packing><property name=\"expand\">Fals"
  "e</property><property name=\"fill\">False</property><property name=\"po"
  "sition\">0</property></packing></child><child><object class=\"GtkLabel\""
  " id=\"fullscreen_preview_label\"><property name=\"visible\">True</prope"
  "rty><property name=\"can_focus\">False</property><property name=\"halig"
  "n\">start</property><property name=\"hexpand\">True</property><property"
  " name=\"label\" translatable=\"yes\">&lt;b&gt;Screensaver preview&lt;/b"
  "&gt;</property><property name=\"use_markup\">True</property><property n"
  "ame=\"justify\">fill</property><property name=\"wrap\">True</property><"
  "/object><packing><property name=\"expand\">False</property><property na"
  "me=\"fill\">True</property><property name=\"position\">1</property></pa"
  "cking></child><child type=\"center\"><object class=\"GtkLabel\" id=\"fu"
  "llscreen_preview_theme_label\"><property name=\"visible\">True</propert"
  "y><property name=\"can_focus\">False</property></object><packing><prope"
  "rty name=\"expand\">True</property><property name=\"fill\">True</proper"
  "ty><property name=\"position\">6</property></packing></child><child><ob"
  "ject class=\"GtkButton\" id=\"fullscreen_preview_close\"><property name"
  "=\"label\">gtk-leave-fullscreen</property><property name=\"visible\">Tr"
  "ue</property><property name=\"can_focus\">True</property><property name"
  "=\"receives_default\">False</property><property name=\"use_stock\">True"
  "</property></object><packing><property name=\"expand\">False</property>"
  "<property name=\"fill\">False</property><property name=\"pack_type\">en"
  "d</property><property name=\"position\">3</property></packing></child><"
  "child><object class=\"GtkBox\" id=\"fullscreen_preview_nav_buttons\"><p"
  "roperty name=\"visible\">True</property><property name=\"can_focus\">Fa"
  "lse</property><child><object class=\"GtkButton\" id=\"fullscreen_previe"
  "w_previous_button\"><property name=\"visible\">True</property><property"
  " name=\"can_focus\">True</property><property name=\"receives_default\">"
  "False</property><child><object class=\"GtkImage\" id=\"image5\"><proper"
  "ty name=\"visible\">True</property><property name=\"can_focus\">False</"
  "property><property name=\"stock\">gtk-go-back</property></object></chil"
  "d></object><packing><property name=\"expand\">False</property><property"
  " name=\"fill\">False</property><property name=\"position\">0</property>"
  "</packing></child><child><object class=\"GtkButton\" id=\"fullscreen_pr"
  "eview_next_button\"><property name=\"visible\">True</property><property"
  " name=\"can_focus\">True</property><property name=\"receives_default\">"
  "False</property><child><object class=\"GtkImage\" id=\"image4\"><proper"
  "ty name=\"visible\">True</property><property name=\"can_focus\">False</"
  "property><property name=\"stock\">gtk-go-forward</property></object></c"
  "hild><style><class name=\"linked\"/></style></object><packing><property"
  " name=\"expand\">False</property><property name=\"fill\">True</property"
  "><property name=\"position\">1</property></packing></child><style><clas"
  "s name=\"linked\"/></style></object><packing><property name=\"expand\">"
  "False</property><property name=\"fill\">False</property><property name="
  "\"pack_type\">end</property><property name=\"position\">4</property></p"
  "acking></child></object><packing><property name=\"expand\">False</prope"
  "rty><property name=\"fill\">False</property><property name=\"position\""
  ">0</property></packing></child><child><object class=\"GtkDrawingArea\" "
  "id=\"fullscreen_preview_area\"><property name=\"visible\">True</propert"
  "y><property name=\"app_paintable\">True</property><property name=\"can_"
  "focus\">False</property></object><packing><property name=\"expand\">Tru"
  "e</property><property name=\"fill\">True</property><property name=\"pos"
  "ition\">1</property></packing></child></object></child></object><object"
  " class=\"GtkAdjustment\" id=\"prefs_dialog_adj_lock_saver_activation_de"
  "lay\"><property name=\"upper\">1440</property><property name=\"step_inc"
  "rement\">1</property><property name=\"page_increment\">10</property></o"
  "bject><object class=\"GtkAdjustment\" id=\"prefs_dialog_adj_logout_dela"
  "y\"><property name=\"upper\">1440</property><property name=\"value\">12"
  "0</property><property name=\"step_increment\">1</property><property nam"
  "e=\"page_increment\">10</property></object><object class=\"GtkAdjustmen"
  "t\" id=\"prefs_dialog_adj_saver_idle_activation_delay\"><property name="
  "\"lower\">1</property><property name=\"upper\">1440</property><property"
  " name=\"value\">5</property><property name=\"step_increment\">1</proper"
  "ty><property name=\"page_increment\">10</property></object><object clas"
  "s=\"GtkAdjustment\" id=\"prefs_dialog_adj_saver_themes_cycle_delay\"><p"
  "roperty name=\"lower\">1</property><property name=\"upper\">1440</prope"
  "rty><property name=\"value\">10</property><property name=\"step_increme"
  "nt\">1</property><property name=\"page_increment\">10</property></objec"
  "t><object class=\"GtkImage\" id=\"prefs_dialog_image_power_management\""
  "><property name=\"can_focus\">False</property><property name=\"stock\">"
  "gtk-jump-to</property></object><object class=\"GtkImage\" id=\"prefs_di"
  "alog_image_preview\"><property name=\"can_focus\">False</property><prop"
  "erty name=\"stock\">gtk-fullscreen</property></object><object class=\"X"
  "fceTitledDialog\" id=\"prefs_dialog\"><property name=\"can_focus\">Fals"
  "e</property><property name=\"title\" translatable=\"yes\">Screensaver P"
  "references</property><property name=\"default_width\">600</property><pr"
  "operty name=\"default_height\">400</property><property name=\"icon_name"
  "\">preferences-desktop-screensaver</property><property name=\"type_hint"
  "\">dialog</property><property name=\"subtitle\" translatable=\"yes\">Co"
  "nfigure your screensaver and locker</property><child internal-child=\"v"
  "box\"><object class=\"GtkBox\" id=\"dialog-vbox1\"><property name=\"vis"
  "ible\">True</property><property name=\"can_focus\">False</property><pro"
  "perty name=\"orientation\">vertical</property><property name=\"spacing\""
  ">2</property><child internal-child=\"action_area\"><object class=\"GtkB"
  "uttonBox\" id=\"dialog-action_area1\"><property name=\"visible\">True</"
  "property><property name=\"can_focus\">False</property><property name=\""
  "hexpand\">True</property><property name=\"layout_style\">end</property>"
  "<child><object class=\"GtkButton\" id=\"help_button\"><property name=\""
  "label\">gtk-help</property><property name=\"visible\">True</property><p"
  "roperty name=\"can_focus\">True</property><property name=\"can_default\""
  ">True</property><property name=\"receives_default\">False</property><pr"
  "operty name=\"use_stock\">True</property></object><packing><property na"
  "me=\"expand\">False</property><property name=\"fill\">False</property><"
  "property name=\"position\">0</property><property name=\"secondary\">Tru"
  "e</property></packing></child><child><object class=\"GtkButton\" id=\"p"
  "ower_management_button\"><property name=\"label\" translatable=\"yes\">"
  "Power _Management</property><property name=\"visible\">True</property><"
  "property name=\"can_focus\">True</property><property name=\"can_default"
  "\">True</property><property name=\"receives_default\">False</property><"
  "property name=\"image\">prefs_dialog_image_power_management</property><"
  "property name=\"use_underline\">True</property></object><packing><prope"
  "rty name=\"expand\">False</property><property name=\"fill\">False</prop"
  "erty><property name=\"position\">1</property><property name=\"secondary"
  "\">True</property></packing></child><child><object class=\"GtkButton\" "
  "id=\"preview_button\"><property name=\"label\" translatable=\"yes\">Pre"
  "view</property><property name=\"visible\">True</property><property name"
  "=\"can_focus\">True</property><property name=\"receives_default\">True<"
  "/property><property name=\"halign\">end</property><property name=\"vali"
  "gn\">center</property><property name=\"image\">prefs_dialog_image_previ"
  "ew</property></object><packing><property name=\"expand\">True</property"
  "><property name=\"fill\">True</property><property name=\"position\">2</"
  "property></packing></child><child><object class=\"GtkButton\" id=\"clos"
  "e_button\"><property name=\"label\">gtk-close</property><property name="
  "\"visible\">True</property><property name=\"can_focus\">True</property>"
  "<property name=\"can_default\">True</property><property name=\"receives"
  "_default\">False</property><property name=\"use_stock\">True</property>"
  "</object><packing><property name=\"expand\">False</property><property n"
  "ame=\"fill\">False</property><property name=\"position\">3</property></"
  "packing></child></object><packing><property name=\"expand\">True</prope"
  "rty><property name=\"fill\">True</property><property name=\"pack_type\""
  ">end</property><property name=\"position\">0</property></packing></chil"
  "d><child><object class=\"GtkBox\" id=\"plug-child\"><property name=\"vi"
  "sible\">True</property><property name=\"can_focus\">False</property><pr"
  "operty name=\"border_width\">6</property><property name=\"orientation\""
  ">vertical</property><property name=\"spacing\">12</property><child><obj"
  "ect class=\"GtkInfoBar\" id=\"root_warning_infobar\"><property name=\"c"
  "an_focus\">False</property><property name=\"no_show_all\">True</propert"
  "y><property name=\"message_type\">warning</property><child internal-chi"
  "ld=\"action_area\"><object class=\"GtkButtonBox\"><property name=\"can_"
  "focus\">False</property><property name=\"spacing\">6</property><propert"
  "y name=\"layout_style\">end</property><child><placeholder/></child></ob"
  "ject><packing><property name=\"expand\">False</property><property name="
  "\"fill\">False</property><property name=\"position\">0</property></pack"
  "ing></child><child internal-child=\"content_area\"><object class=\"GtkB"
  "ox\"><property name=\"can_focus\">False</property><property name=\"spac"
  "ing\">16</property><child><object class=\"GtkImage\"><property name=\"v"
  "isible\">True</property><property name=\"can_focus\">False</property><p"
  "roperty name=\"pixel_size\">24</property><property name=\"icon_name\">d"
  "ialog-warning</property><property name=\"icon_size\">3</property></obje"
  "ct><packing><property name=\"expand\">False</property><property name=\""
  "fill\">True</property><property name=\"position\">0</property></packing"
  "></child><child><object class=\"GtkBox\"><property name=\"visible\">Tru"
  "e</property><property name=\"can_focus\">False</property><property name"
  "=\"orientation\">vertical</property><child><object class=\"GtkLabel\"><"
  "property name=\"visible\">True</property><property name=\"can_focus\">F"
  "alse</property><property name=\"label\" translatable=\"yes\">Running as"
  " root</property><property name=\"xalign\">0</property><attributes><attr"
  "ibute name=\"weight\" value=\"bold\"/></attributes></object><packing><p"
  "roperty name=\"expand\">False</property><property name=\"fill\">True</p"
  "roperty><property name=\"position\">0</property></packing></child><chil"
  "d><object class=\"GtkLabel\"><property name=\"visible\">True</property>"
  "<property name=\"can_focus\">False</property><property name=\"label\" t"
  "ranslatable=\"yes\">The screen will not be locked for the root user.</p"
  "roperty><property name=\"xalign\">0</property></object><packing><proper"
  "ty name=\"expand\">False</property><property name=\"fill\">True</proper"
  "ty><property name=\"position\">1</property></packing></child></object><"
  "packing><property name=\"expand\">False</property><property name=\"fill"
  "\">True</property><property name=\"position\">1</property></packing></c"
  "hild></object><packing><property name=\"expand\">False</property><prope"
  "rty name=\"fill\">False</property><property name=\"position\">0</proper"
  "ty></packing></child></object><packing><property name=\"expand\">False<"
  "/property><property name=\"fill\">True</property><property name=\"posit"
  "ion\">0</property></packing></child><child><object class=\"GtkInfoBar\""
  " id=\"logind_lid_infobar\"><property name=\"can_focus\">False</property"
  "><property name=\"no_show_all\">True</property><child internal-child=\""
  "action_area\"><object class=\"GtkButtonBox\"><property name=\"can_focus"
  "\">False</property><property name=\"spacing\">6</property><property nam"
  "e=\"layout_style\">end</property><child><object class=\"GtkButton\" id="
  "\"logind_lid_resolve\"><property name=\"label\" translatable=\"yes\">Re"
  "solve</property><property name=\"visible\">True</property><property nam"
  "e=\"can_focus\">True</property><property name=\"receives_default\">True"
  "</property></object><packing><property name=\"expand\">True</property><"
  "property name=\"fill\">True</property><property name=\"position\">0</pr"
  "operty></packing></child></object><packing><property name=\"expand\">Fa"
  "lse</property><property name=\"fill\">False</property><property name=\""
  "position\">0</property></packing></child><child internal-child=\"conten"
  "t_area\"><object class=\"GtkBox\"><property name=\"can_focus\">False</p"
  "roperty><property name=\"spacing\">16</property><child><object class=\""
  "GtkImage\"><property name=\"visible\">True</property><property name=\"c"
  "an_focus\">False</property><property name=\"pixel_size\">24</property><"
  "property name=\"icon_name\">dialog-information</property><property name"
  "=\"icon_size\">3</property></object><packing><property name=\"expand\">"
  "False</property><property name=\"fill\">True</property><property name=\""
  "position\">0</property></packing></child><child><object class=\"GtkBox\""
  "><property name=\"visible\">True</property><property name=\"can_focus\""
  ">False</property><property name=\"valign\">center</property><property n"
  "ame=\"orientation\">vertical</property><child><object class=\"GtkLabel\""
  "><property name=\"visible\">True</property><property name=\"can_focus\""
  ">False</property><property name=\"label\" translatable=\"yes\">Xfce Pow"
  "er Manager is not configured to handle laptop lid events</property><pro"
  "perty name=\"xalign\">0</property><attributes><attribute name=\"weight\""
  " value=\"bold\"/></attributes></object><packing><property name=\"expand"
  "\">False</property><property name=\"fill\">True</property><property nam"
  "e=\"position\">0</property></packing></child><child><object class=\"Gtk"
  "Label\"><property name=\"visible\">True</property><property name=\"can_"
  "focus\">False</property><property name=\"label\" translatable=\"yes\">Y"
  "our computer may not be locked when you close the lid.</property><prope"
  "rty name=\"xalign\">0</property></object><packing><property name=\"expa"
  "nd\">False</property><property name=\"fill\">True</property><property n"
  "ame=\"position\">1</property></packing></child></object><packing><prope"
  "rty name=\"expand\">False</property><property name=\"fill\">True</prope"
  "rty><property name=\"position\">1</property></packing></child></object>"
  "<packing><property name=\"expand\">False</property><property name=\"fil"
  "l\">False</property><property name=\"position\">0</property></packing><"
  "/child></object><packing><property name=\"expand\">False</property><pro"
  "perty name=\"fill\">True</property><property name=\"position\">1</prope"
  "rty></packing></child><child><object class=\"GtkNotebook\" id=\"noteboo"
  "k\"><property name=\"visible\">True</property><property name=\"can_focu"
  "s\">True</property><child><object class=\"GtkBox\" id=\"saver_page\"><p"
  "roperty name=\"visible\">True</property><property name=\"can_focus\">Fa"
  "lse</property><property name=\"border_width\">12</property><property na"
  "me=\"orientation\">vertical</property><property name=\"spacing\">12</pr"
  "operty><child><object class=\"GtkBox\" id=\"saver_enable_box\"><propert"
  "y name=\"visible\">True</property><property name=\"can_focus\">False</p"
  "roperty><child><object class=\"GtkLabel\" id=\"saver_enable_label\"><pr"
  "operty name=\"visible\">True</property><property name=\"can_focus\">Fal"
  "se</property><property name=\"label\" translatable=\"yes\">Enable Scree"
  "nsaver</property><property name=\"xalign\">0</property><attributes><att"
  "ribute name=\"weight\" value=\"bold\"/></attributes></object><packing><"
  "property name=\"expand\">False</property><property name=\"fill\">True</"
  "property><property name=\"position\">0</property></packing></child><chi"
  "ld><object class=\"GtkSwitch\" id=\"saver_enabled\"><property name=\"vi"
  "sible\">True</property><property name=\"can_focus\">True</property><pro"
  "perty name=\"halign\">end</property><property name=\"valign\">center</p"
  "roperty><property name=\"active\">True</property></object><packing><pro"
  "perty name=\"expand\">False</property><property name=\"fill\">True</pro"
  "perty><property name=\"pack_type\">end</property><property name=\"posit"
  "ion\">1</property></packing></child></object><packing><property name=\""
  "expand\">False</property><property name=\"fill\">True</property><proper"
  "ty name=\"position\">0</property></packing></child><child><object class"
  "=\"GtkGrid\" id=\"saver_grid\"><property name=\"visible\">True</propert"
  "y><property name=\"can_focus\">False</property><property name=\"vexpand"
  "\">True</property><property name=\"row_spacing\">6</property><property "
  "name=\"column_spacing\">6</property><child><object class=\"GtkGrid\" id"
  "=\"saver_themes_grid\"><property name=\"visible\">True</property><prope"
  "rty name=\"can_focus\">False</property><property name=\"margin_top\">14"
  "</property><property name=\"margin_bottom\">6</property><property name="
  "\"vexpand\">True</property><property name=\"row_spacing\">6</property><"
  "property name=\"column_spacing\">12</property><child><object class=\"Gt"
  "kLabel\" id=\"saver_themes_theme_label\"><property name=\"visible\">Tru"
  "e</property><property name=\"can_focus\">False</property><property name"
  "=\"label\" translatable=\"yes\">Theme</property><property name=\"xalign"
  "\">0</property><property name=\"yalign\">1</property><attributes><attri"
  "bute name=\"weight\" value=\"bold\"/></attributes></object><packing><pr"
  "operty name=\"left_attach\">0</property><property name=\"top_attach\">0"
  "</property></packing></child><child><object class=\"GtkLabel\" id=\"sav"
  "er_themes_preview_label\"><property name=\"visible\">True</property><pr"
  "operty name=\"can_focus\">False</property><property name=\"label\" tran"
  "slatable=\"yes\">Preview</property><property name=\"xalign\">0</propert"
  "y><property name=\"yalign\">1</property><attributes><attribute name=\"w"
  "eight\" value=\"bold\"/></attributes></object><packing><property name=\""
  "left_attach\">1</property><property name=\"top_attach\">0</property><pr"
  "operty name=\"width\">2</property></packing></child><child><object clas"
  "s=\"GtkFrame\"><property name=\"visible\">True</property><property name"
  "=\"can_focus\">False</property><property name=\"hexpand\">True</propert"
  "y><property name=\"label_xalign\">0</property><property name=\"shadow_t"
  "ype\">in</property><child><object class=\"GtkDrawingArea\" id=\"saver_t"
  "hemes_preview_area\"><property name=\"app_paintable\">True</property><p"
  "roperty name=\"can_focus\">False</property><property name=\"hexpand\">T"
  "rue</property><property name=\"vexpand\">True</property></object></chil"
  "d><child type=\"label_item\"><placeholder/></child></object><packing><p"
  "roperty name=\"left_attach\">1</property><property name=\"top_attach\">"
  "1</property><property name=\"width\">2</property></packing></child><chi"
  "ld><object class=\"GtkBox\"><property name=\"visible\">True</property><"
  "property name=\"can_focus\">False</property><property name=\"vexpand\">"
  "True</property><property name=\"orientation\">vertical</property><child"
  "><object class=\"GtkScrolledWindow\" id=\"saver_themes_scrolled_window\""
  "><property name=\"visible\">True</property><property name=\"can_focus\""
  ">True</property><property name=\"vexpand\">True</property><property nam"
  "e=\"hscrollbar_policy\">never</property><property name=\"shadow_type\">"
  "in</property><child><object class=\"GtkTreeView\" id=\"saver_themes_tre"
  "eview\"><property name=\"visible\">True</property><property name=\"can_"
  "focus\">True</property><property name=\"headers_visible\">False</proper"
  "ty><child internal-child=\"selection\"><object class=\"GtkTreeSelection"
  "\"/></child><accessibility><relation type=\"label-for\" target=\"saver_"
  "themes_treeview\"/></accessibility></object></child></object><packing><"
  "property name=\"expand\">True</property><property name=\"fill\">True</p"
  "roperty><property name=\"position\">0</property></packing></child><chil"
  "d><object class=\"GtkToolbar\" id=\"configure_toolbar\"><property name="
  "\"visible\">True</property><property name=\"can_focus\">False</property"
  "><property name=\"toolbar_style\">both-horiz</property><property name=\""
  "icon_size\">2</property><child><object class=\"GtkToolButton\" id=\"con"
  "figure_button\"><property name=\"visible\">True</property><property nam"
  "e=\"sensitive\">False</property><property name=\"can_focus\">False</pro"
  "perty><property name=\"tooltip_text\" translatable=\"yes\">Configure sc"
  "reensaver</property><property name=\"label\" translatable=\"yes\">Confi"
  "gure...</property><property name=\"use_underline\">True</property><prop"
  "erty name=\"icon_name\">document-properties</property><style><class nam"
  "e=\"linked\"/></style></object><packing><property name=\"expand\">False"
  "</property><property name=\"homogeneous\">True</property></packing></ch"
  "ild><style><class name=\"inline-toolbar\"/></style></object><packing><p"
  "roperty name=\"expand\">False</property><property name=\"fill\">True</p"
  "roperty><property name=\"position\">1</property></packing></child></obj"
  "ect><packing><property name=\"left_attach\">0</property><property name="
  "\"top_attach\">1</property></packing></child></object><packing><propert"
  "y name=\"left_attach\">0</property><property name=\"top_attach\">0</pro"
  "perty><property name=\"width\">3</property></packing></child><child><ob"
  "ject class=\"GtkLabel\" id=\"saver_themes_cycle_delay_label_left\"><pro"
  "perty name=\"visible\">True</property><property name=\"can_focus\">Fals"
  "e</property><property name=\"hexpand\">True</property><property name=\""
  "label\" translatable=\"yes\">Change theme after:</property><property na"
  "me=\"xalign\">0</property></object><packing><property name=\"left_attac"
  "h\">0</property><property name=\"top_attach\">1</property></packing></c"
  "hild><child><object class=\"GtkSpinButton\" id=\"saver_themes_cycle_del"
  "ay\"><property name=\"visible\">True</property><property name=\"can_foc"
  "us\">True</property><property name=\"adjustment\">prefs_dialog_adj_save"
  "r_themes_cycle_delay</property></object><packing><property name=\"left_"
  "attach\">1</property><property name=\"top_attach\">1</property></packin"
  "g></child><child><object class=\"GtkLabel\" id=\"saver_themes_cycle_del"
  "ay_label_right\"><property name=\"visible\">True</property><property na"
  "me=\"can_focus\">False</property><property name=\"label\" translatable="
  "\"yes\">minutes</property><attributes><attribute name=\"style\" value=\""
  "italic\"/></attributes></object><packing><property name=\"left_attach\""
  ">2</property><property name=\"top_attach\">1</property></packing></chil"
  "d><child><object class=\"GtkLabel\" id=\"saver_idle_activation_label\">"
  "<property name=\"visible\">True</property><property name=\"can_focus\">"
  "False</property><property name=\"margin_top\">12</property><property na"
  "me=\"label\" translatable=\"yes\">Activate screensaver when computer is"
  " idle</property><property name=\"xalign\">0</property><attributes><attr"
  "ibute name=\"weight\" value=\"bold\"/></attributes></object><packing><p"
  "roperty name=\"left_attach\">0</property><property name=\"top_attach\">"
  "2</property></packing></child><child><object class=\"GtkSwitch\" id=\"s"
  "aver_idle_activation_enabled\"><property name=\"visible\">True</propert"
  "y><property name=\"can_focus\">True</property><property name=\"halign\""
  ">end</property><property name=\"valign\">center</property><property nam"
  "e=\"margin_top\">12</property></object><packing><property name=\"left_a"
  "ttach\">1</property><property name=\"top_attach\">2</property><property"
  " name=\"width\">2</property></packing></child><child><object class=\"Gt"
  "kLabel\" id=\"saver_idle_activation_delay_label_left\"><property name=\""
  "visible\">True</property><property name=\"can_focus\">False</property><"
  "property name=\"margin_left\">12</property><property name=\"hexpand\">T"
  "rue</property><property name=\"label\" translatable=\"yes\">Regard the "
  "computer as _idle after:</property><property name=\"use_underline\">Tru"
  "e</property><property name=\"xalign\">0</property></object><packing><pr"
  "operty name=\"left_attach\">0</property><property name=\"top_attach\">3"
  "</property></packing></child><child><object class=\"GtkSpinButton\" id="
  "\"saver_idle_activation_delay\"><property name=\"visible\">True</proper"
  "ty><property name=\"can_focus\">True</property><property name=\"adjustm"
  "ent\">prefs_dialog_adj_saver_idle_activation_delay</property></object><"
  "packing><property name=\"left_attach\">1</property><property name=\"top"
  "_attach\">3</property></packing></child><child><object class=\"GtkLabel"
  "\" id=\"saver_idle_activation_delay_label_right\"><property name=\"visi"
  "ble\">True</property><property name=\"can_focus\">False</property><prop"
  "erty name=\"label\" translatable=\"yes\">minutes</property><attributes>"
  "<attribute name=\"style\" value=\"italic\"/></attributes></object><pack"
  "ing><property name=\"left_attach\">2</property><property name=\"top_att"
  "ach\">3</property></packing></child><child><object class=\"GtkLabel\" i"
  "d=\"saver_fullscreen_inhibit_label\"><property name=\"visible\">True</p"
  "roperty><property name=\"can_focus\">False</property><property name=\"m"
  "argin_top\">12</property><property name=\"label\" translatable=\"yes\">"
  "Inhibit screensaver for fullscreen applications</property><property nam"
  "e=\"xalign\">0</property><attributes><attribute name=\"weight\" value=\""
  "bold\"/></attributes></object><packing><property name=\"left_attach\">0"
  "</property><property name=\"top_attach\">4</property></packing></child>"
  "<child><object class=\"GtkSwitch\" id=\"saver_fullscreen_inhibit_enable"
  "d\"><property name=\"visible\">True</property><property name=\"can_focu"
  "s\">True</property><property name=\"halign\">end</property><property na"
  "me=\"valign\">center</property><property name=\"margin_top\">12</proper"
  "ty></object><packing><property name=\"left_attach\">1</property><proper"
  "ty name=\"top_attach\">4</property><property name=\"width\">2</property"
  "></packing></child></object><packing><property name=\"expand\">False</p"
  "roperty><property name=\"fill\">True</property><property name=\"positio"
  "n\">1</property></packing></child></object></child><child type=\"tab\">"
  "<object class=\"GtkLabel\" id=\"saver_page_label\"><property name=\"vis"
  "ible\">True</property><property name=\"can_focus\">False</property><pro"
  "perty name=\"label\" translatable=\"yes\">Screensaver</property></objec"
  "t><packing><property name=\"tab_fill\">False</property></packing></chil"
  "d><child><object class=\"GtkBox\" id=\"lock_page\"><property name=\"vis"
  "ible\">True</property><property name=\"can_focus\">False</property><pro"
  "perty name=\"border_width\">12</property><property name=\"orientation\""
  ">vertical</property><property name=\"spacing\">12</property><child><obj"
  "ect class=\"GtkBox\" id=\"lock_enable_box\"><property name=\"visible\">"
  "True</property><property name=\"can_focus\">False</property><child><obj"
  "ect class=\"GtkLabel\" id=\"lock_enable_label\"><property name=\"visibl"
  "e\">True</property><property name=\"can_focus\">False</property><proper"
  "ty name=\"label\" translatable=\"yes\">Enable Lock Screen</property><at"
  "tributes><attribute name=\"weight\" value=\"bold\"/></attributes></obje"
  "ct><packing><property name=\"expand\">False</property><property name=\""
  "fill\">True</property><property name=\"position\">0</property></packing"
  "></child><child><object class=\"GtkSwitch\" id=\"lock_enabled\"><proper"
  "ty name=\"visible\">True</property><property name=\"can_focus\">True</p"
  "roperty><property name=\"halign\">end</property><property name=\"valign"
  "\">center</property><property name=\"active\">True</property></object><"
  "packing><property name=\"expand\">False</property><property name=\"fill"
  "\">True</property><property name=\"pack_type\">end</property><property "
  "name=\"position\">1</property></packing></child></object><packing><prop"
  "erty name=\"expand\">False</property><property name=\"fill\">True</prop"
  "erty><property name=\"position\">0</property></packing></child><child><"
  "object class=\"GtkGrid\" id=\"lock_grid\"><property name=\"visible\">Tr"
  "ue</property><property name=\"can_focus\">False</property><property nam"
  "e=\"row_spacing\">6</property><property name=\"column_spacing\">6</prop"
  "erty><child><object class=\"GtkLabel\" id=\"lock_saver_activation_label"
  "\"><property name=\"visible\">True</property><property name=\"can_focus"
  "\">False</property><property name=\"margin_top\">12</property><property"
  " name=\"label\" translatable=\"yes\">Lock Screen with Screensaver</prop"
  "erty><property name=\"xalign\">0</property><attributes><attribute name="
  "\"weight\" value=\"bold\"/></attributes></object><packing><property nam"
  "e=\"left_attach\">0</property><property name=\"top_attach\">0</property"
  "><property name=\"width\">2</property></packing></child><child><object "
  "class=\"GtkSwitch\" id=\"lock_saver_activation_enabled\"><property name"
  "=\"visible\">True</property><property name=\"can_focus\">True</property"
  "><property name=\"halign\">end</property><property name=\"valign\">cent"
  "er</property><property name=\"margin_top\">12</property></object><packi"
  "ng><property name=\"left_attach\">2</property><property name=\"top_atta"
  "ch\">0</property></packing></child><child><object class=\"GtkLabel\" id"
  "=\"lock_saver_activation_delay_label_left\"><property name=\"visible\">"
  "True</property><property name=\"can_focus\">False</property><property n"
  "ame=\"margin_left\">12</property><property name=\"label\" translatable="
  "\"yes\">Lock the screen after the screensaver is active for:</property>"
  "<property name=\"xalign\">0</property></object><packing><property name="
  "\"left_attach\">0</property><property name=\"top_attach\">1</property><"
  "/packing></child><child><object class=\"GtkSpinButton\" id=\"lock_saver"
  "_activation_delay\"><property name=\"visible\">True</property><property"
  " name=\"can_focus\">True</property><property name=\"adjustment\">prefs_"
  "dialog_adj_lock_saver_activation_delay</property></object><packing><pro"
  "perty name=\"left_attach\">1</property><property name=\"top_attach\">1<"
  "/property></packing></child><child><object class=\"GtkLabel\" id=\"lock"
  "_saver_activation_delay_label_right\"><property name=\"visible\">True</"
  "property><property name=\"can_focus\">False</property><property name=\""
  "label\" translatable=\"yes\">minutes</property><attributes><attribute n"
  "ame=\"style\" value=\"italic\"/></attributes></object><packing><propert"
  "y name=\"left_attach\">2</property><property name=\"top_attach\">1</pro"
  "perty></packing></child><child><object class=\"GtkLabel\" id=\"lock_emb"
  "edded_keyboard_label\"><property name=\"visible\">True</property><prope"
  "rty name=\"can_focus\">False</property><property name=\"margin_top\">12"
  "</property><property name=\"label\" translatable=\"yes\">On Screen Keyb"
  "oard</property><property name=\"xalign\">0</property><attributes><attri"
  "bute name=\"weight\" value=\"bold\"/></attributes></object><packing><pr"
  "operty name=\"left_attach\">0</property><property name=\"top_attach\">3"
  "</property><property name=\"width\">2</property></packing></child><chil"
  "d><object class=\"GtkSwitch\" id=\"lock_embedded_keyboard_enabled\"><pr"
  "operty name=\"visible\">True</property><property name=\"can_focus\">Tru"
  "e</property><property name=\"halign\">end</property><property name=\"va"
  "lign\">center</property><property name=\"margin_top\">12</property></ob"
  "ject><packing><property name=\"left_attach\">2</property><property name"
  "=\"top_attach\">3</property></packing></child><child><object class=\"Gt"
  "kLabel\" id=\"lock_embedded_keyboard_command_label\"><property name=\"v"
  "isible\">True</property><property name=\"can_focus\">False</property><p"
  "roperty name=\"margin_left\">12</property><property name=\"label\" tran"
  "slatable=\"yes\">On screen keyboard command:</property><property name=\""
  "xalign\">0</property></object><packing><property name=\"left_attach\">0"
  "</property><property name=\"top_attach\">4</property></packing></child>"
  "<child><object class=\"GtkEntry\" id=\"lock_embedded_keyboard_command\""
  "><property name=\"visible\">True</property><property name=\"can_focus\""
  ">True</property></object><packing><property name=\"left_attach\">1</pro"
  "perty><property name=\"top_attach\">4</property><property name=\"width\""
  ">2</property></packing></child><child><object class=\"GtkLabel\" id=\"l"
  "ock_status_messages_label\"><property name=\"visible\">True</property><"
  "property name=\"can_focus\">False</property><property name=\"margin_top"
  "\">12</property><property name=\"hexpand\">True</property><property nam"
  "e=\"label\" translatable=\"yes\">Session Status Messages</property><pro"
  "perty name=\"xalign\">0</property><attributes><attribute name=\"weight\""
  " value=\"bold\"/></attributes></object><packing><property name=\"left_a"
  "ttach\">0</property><property name=\"top_attach\">5</property></packing"
  "></child><child><object class=\"GtkSwitch\" id=\"lock_status_messages_e"
  "nabled\"><property name=\"visible\">True</property><property name=\"can"
  "_focus\">True</property><property name=\"halign\">end</property><proper"
  "ty name=\"valign\">center</property><property name=\"margin_top\">12</p"
  "roperty></object><packing><property name=\"left_attach\">1</property><p"
  "roperty name=\"top_attach\">5</property><property name=\"width\">2</pro"
  "perty></packing></child><child><object class=\"GtkLabel\" id=\"lock_log"
  "out_label\"><property name=\"visible\">True</property><property name=\""
  "can_focus\">False</property><property name=\"margin_top\">12</property>"
  "<property name=\"label\" translatable=\"yes\">Logout</property><propert"
  "y name=\"xalign\">0</property><attributes><attribute name=\"weight\" va"
  "lue=\"bold\"/></attributes></object><packing><property name=\"left_atta"
  "ch\">0</property><property name=\"top_attach\">6</property><property na"
  "me=\"width\">2</property></packing></child><child><object class=\"GtkSw"
  "itch\" id=\"lock_logout_enabled\"><property name=\"visible\">True</prop"
  "erty><property name=\"can_focus\">True</property><property name=\"halig"
  "n\">end</property><property name=\"valign\">center</property><property "
  "name=\"margin_top\">12</property></object><packing><property name=\"lef"
  "t_attach\">2</property><property name=\"top_attach\">6</property></pack"
  "ing></child><child><object class=\"GtkLabel\" id=\"lock_logout_command_"
  "label\"><property name=\"visible\">True</property><property name=\"can_"
  "focus\">False</property><property name=\"margin_left\">12</property><pr"
  "operty name=\"label\" translatable=\"yes\">Logout command:</property><p"
  "roperty name=\"xalign\">0</property></object><packing><property name=\""
  "left_attach\">0</property><property name=\"top_attach\">7</property></p"
  "acking></child><child><object class=\"GtkEntry\" id=\"lock_logout_comma"
  "nd\"><property name=\"visible\">True</property><property name=\"can_foc"
  "us\">True</property></object><packing><property name=\"left_attach\">1<"
  "/property><property name=\"top_attach\">7</property><property name=\"wi"
  "dth\">2</property></packing></child><child><object class=\"GtkLabel\" i"
  "d=\"lock_logout_delay_label_left\"><property name=\"visible\">True</pro"
  "perty><property name=\"can_focus\">False</property><property name=\"mar"
  "gin_left\">12</property><property name=\"label\" translatable=\"yes\">E"
  "nable logout after:</property><property name=\"xalign\">0</property></o"
  "bject><packing><property name=\"left_attach\">0</property><property nam"
  "e=\"top_attach\">8</property></packing></child><child><object class=\"G"
  "tkSpinButton\" id=\"lock_logout_delay\"><property name=\"visible\">True"
  "</property><property name=\"can_focus\">True</property><property name=\""
  "adjustment\">prefs_dialog_adj_logout_delay</property></object><packing>"
  "<property name=\"left_attach\">1</property><property name=\"top_attach\""
  ">8</property></packing></child><child><object class=\"GtkLabel\" id=\"l"
  "ock_logout_delay_label_right\"><property name=\"visible\">True</propert"
  "y><property name=\"can_focus\">False</property><property name=\"label\""
  " translatable=\"yes\">minutes</property><attributes><attribute name=\"s"
  "tyle\" value=\"italic\"/></attributes></object><packing><property name="
  "\"left_attach\">2</property><property name=\"top_attach\">8</property><"
  "/packing></child><child><object class=\"GtkLabel\" id=\"lock_user_switc"
  "hing_label\"><property name=\"visible\">True</property><property name=\""
  "can_focus\">False</property><property name=\"margin_top\">12</property>"
  "<property name=\"label\" translatable=\"yes\">User Switching</property>"
  "<property name=\"xalign\">0</property><attributes><attribute name=\"wei"
  "ght\" value=\"bold\"/></attributes></object><packing><property name=\"l"
  "eft_attach\">0</property><property name=\"top_attach\">9</property><pro"
  "perty name=\"width\">2</property></packing></child><child><object class"
  "=\"GtkSwitch\" id=\"lock_user_switching_enabled\"><property name=\"visi"
  "ble\">True</property><property name=\"can_focus\">True</property><prope"
  "rty name=\"halign\">end</property><property name=\"valign\">center</pro"
  "perty><property name=\"margin_top\">12</property></object><packing><pro"
  "perty name=\"left_attach\">2</property><property name=\"top_attach\">9<"
  "/property></packing></child><child><object class=\"GtkLabel\" id=\"lock"
  "_suspend_hibernate_label\"><property name=\"visible\">True</property><p"
  "roperty name=\"can_focus\">False</property><property name=\"margin_top\""
  ">12</property><property name=\"label\" translatable=\"yes\">Lock Screen"
  " with System Sleep</property><property name=\"xalign\">0</property><att"
  "ributes><attribute name=\"weight\" value=\"bold\"/></attributes></objec"
  "t><packing><property name=\"left_attach\">0</property><property name=\""
  "top_attach\">2</property><property name=\"width\">2</property></packing"
  "></child><child><object class=\"GtkSwitch\" id=\"lock_suspend_hibernate"
  "_enabled\"><property name=\"visible\">True</property><property name=\"c"
  "an_focus\">True</property><property name=\"halign\">end</property><prop"
  "erty name=\"valign\">center</property><property name=\"margin_top\">12<"
  "/property></object><packing><property name=\"left_attach\">2</property>"
  "<property name=\"top_attach\">2</property></packing></child></object><p"
  "acking><property name=\"expand\">False</property><property name=\"fill\""
  ">True</property><property name=\"position\">1</property></packing></chi"
  "ld></object><packing><property name=\"position\">1</property></packing>"
  "</child><child type=\"tab\"><object class=\"GtkLabel\" id=\"lock_page_l"
  "abel\"><property name=\"visible\">True</property><property name=\"can_f"
  "ocus\">False</property><property name=\"label\" translatable=\"yes\">Lo"
  "ck Screen</property></object><packing><property name=\"position\">1</pr"
  "operty><property name=\"tab_fill\">False</property></packing></child><c"
  "hild><placeholder/></child><child type=\"tab\"><placeholder/></child></"
  "object><packing><property name=\"expand\">True</property><property name"
  "=\"fill\">True</property><property name=\"position\">4</property></pack"
  "ing></child></object><packing><property name=\"expand\">True</property>"
  "<property name=\"fill\">True</property><property name=\"position\">1</p"
  "roperty></packing></child></object></child><action-widgets><action-widg"
  "et response=\"-11\">help_button</action-widget><action-widget response="
  "\"-2\">power_management_button</action-widget><action-widget response=\""
  "-7\">close_button</action-widget></action-widgets></object></interface>"
  ""
};

static const unsigned xfce4_screensaver_preferences_ui_length = 38170u;

