/* automatically generated from xfce4-screensaver-dialog.css */
#ifdef __SUNPRO_C
#pragma align 4 (xfce4_screensaver_dialog_css)
#endif
#ifdef __GNUC__
static const char xfce4_screensaver_dialog_css[] __attribute__ ((__aligned__ (4))) =
#else
static const char xfce4_screensaver_dialog_css[] =
#endif
{
  "/* Workaround for Adwaita - and other themes - setting a smaller font f"
  "or the keycap window */\n#login_window.keycap { font-size: initial; }\n"
};

static const unsigned xfce4_screensaver_dialog_css_length = 140u;

